==> blender-2.47.tar.gz: blender-2.47/source/kernel/gen_system/GEN_HashedPtr.cpp <==
/*
 * $Id: GEN_HashedPtr.cpp 14444 2008-04-16 22:40:48Z hos $
 *
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 * Contributor(s): none yet.
 *
 * ***** END GPL LICENSE BLOCK *****
 *
 */
#include "GEN_HashedPtr.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//
// Build hash index from pointer.  Even though the final result
// is a 32-bit integer, use all the bits of the pointer as long
// as possible.
//

unsigned int GEN_Hash(void * inDWord)
{
#if defined(_WIN64)
	unsigned __int64 key = (unsigned __int64)inDWord;
#else
	unsigned long key = (unsigned long)inDWord;
#endif

	key += ~(key << 16);
	key ^=  (key >>  5);
	key +=  (key <<  3);
	key ^=  (key >> 13);
	key += ~(key <<  9);
	key ^=  (key >> 17);

  	return (unsigned int)(key & 0xffffffff);
}

==> blender-2.48a.tar.gz: blender-2.48a/source/kernel/gen_system/GEN_HashedPtr.cpp <==
/*
 * $Id: GEN_HashedPtr.cpp 16147 2008-08-17 17:08:00Z genscher $
 *
 * ***** BEGIN GPL LICENSE BLOCK *****
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
 * All rights reserved.
 *
 * The Original Code is: all of this file.
 *
 * Contributor(s): none yet.
 *
 * ***** END GPL LICENSE BLOCK *****
 *
 */
#include "GEN_HashedPtr.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "BLO_sys_types.h" // for intptr_t support

//
// Build hash index from pointer.  Even though the final result
// is a 32-bit integer, use all the bits of the pointer as long
// as possible.
//

unsigned int GEN_Hash(void * inDWord)
{
	uintptr_t key = (uintptr_t)inDWord;

	key += ~(key << 16);
	key ^=  (key >>  5);
	key +=  (key <<  3);
	key ^=  (key >> 13);
	key += ~(key <<  9);
	key ^=  (key >> 17);

  	return (unsigned int)(key & 0xffffffff);
}
